@@ -17,7 +17,7 @@ from torchaudio.datasets.utils import (
     download_url,
     extract_archive,
 )
-
+import hashlib
 
 
 _CHECKSUMS = {"mls_english_opus": "60390221eec6f456611563b37f0b052c",
@@ -56,7 +56,14 @@ espeak_map = {"mls_english_opus": "en",
               "mls_portuguese": "pt-pt",
               "mls_polish":  "pl"}
     
+def check_md5(path, md5_hash):
+    with open(path, 'rb') as file_to_check:
+        # read contents of the file
+        data = file_to_check.read()    
+        # pipe contents of the file through
+        md5_returned = hashlib.md5(data).hexdigest()
 
+        assert md5_returned==md5_hash, f"{os.path.basename(path)} is corrupted, please download it again"
 
 class MultilingualLibriSpeech(Dataset):
     """Dataset class for Multilingual LibriSpeech (MLS) dataset.
@@ -438,10 +445,9 @@ class TIMIT(Dataset):
             ``(default: ``"mls_german_opus"``).
             
         split (str):
-            Choose different dataset splits such as ``"train"``, ``"dev"``, and
-            ``"test"``. (default: ``"train"``)
+            Choose different dataset splits such as ``"train"``, or ``"test"``. (default: ``"train"``).
+            ``"dev"`` is not given in this dataset, you have to split part of the ``"train"`` into the ``"dev"`` set.
 
-        low_resource: bool = False,
         
         download (bool, optional):
             Whether to download the dataset if it is not found at root path. (default: ``False``).
@@ -463,25 +469,34 @@ class TIMIT(Dataset):
 
         self._ext_audio = ".wav" # The audio format in this dataset is .wav
         ext_archive = '.zip'
-        url = "https://www.kaggle.com/mfekadu/darpa-timit-acousticphonetic-continuous-speech/download"
+#         url = "https://www.kaggle.com/mfekadu/darpa-timit-acousticphonetic-continuous-speech/download"
+        url = 'https://data.deepai.org/timit.zip'
         
         # Getting audio path
-        archive_name = 'archive'
+        archive_name = 'timit'
         folder_name = 'TIMIT'
         download_path = os.path.join(root, folder_name)
         self.download_path = download_path
+        assert split.upper()=="TRAIN" or split.upper()=="TEST", f"{split=} is not present in this dataset"
         self._path = os.path.join(root, folder_name, 'data', split.upper())
         
         self.groups = self.available_groups(groups)
         
-        checksum = '8acbaea404859a04925b5bd32474d3ce'
+        checksum = '5b736303c55cf4970926bb9978b655fe'
         
         if download:
             if os.path.isdir(download_path) and os.path.isdir(os.path.join(download_path, 'data')):
                 print(f'Dataset folder exists, skipping download...')
                 decision = input(f"Do you want to extract {archive_name+ext_archive} again? "
                                  f"To avoid this prompt, set `download=False`\n"
-                                 f"This action will overwrite exsiting files, do you still want to continue? [yes/no]")                
+                                 f"This action will overwrite exsiting files, do you still want to continue? [yes/no]") 
+                if decision.lower()=='yes':
+                    print(f'extracting...')
+                    extract_archive(os.path.join(download_path, archive_name+ext_archive))                
+            elif os.path.isfile(os.path.join(download_path, 'timit.zip')):
+                print(f'timit.zip exists, extracting...')
+                check_md5(os.path.join(download_path, archive_name+ext_archive), checksum)
+                extract_archive(os.path.join(download_path, archive_name+ext_archive))
             else:
                 decision='yes'       
                 if not os.path.isdir(download_path):
@@ -501,14 +516,15 @@ class TIMIT(Dataset):
                                         'You may want to download it manually from:\n'+
                                         url+ '\n' +
                                         f'Then, put it inside {download_path}')
-                    
-            if decision.lower()=='yes':
-                print(f'extracting...')
-                extract_archive(os.path.join(download_path, archive_name+ext_archive))
 
         
         if os.path.isdir(self._path):
             pass
+        elif os.path.isfile(os.path.join(download_path, 'timit.zip')):
+            print(f'timit.zip exists, extracting...')
+            check_md5(os.path.join(download_path, archive_name+ext_archive), checksum)
+            extract_archive(os.path.join(download_path, archive_name+ext_archive))
+            
         else:
             raise FileNotFoundError(f"Dataset not found at {self._path}, please specify the correct location or set `download=True`")
                     
@@ -560,11 +576,14 @@ class TIMIT(Dataset):
         ext: either `'WRD` for word labels or `PHN` for phone labels
         """
         label_path = file_path.replace('WAV.wav', ext)
-        labels = []
+        labels = ''
         with open(label_path, 'r') as f:
             lines = f.read().splitlines()
             for i in lines:
-                labels.append(i.split(' ')[-1]) 
+                labels += f"{(i.split(' ')[-1])} "
+            if ext=='PHN':
+                # Remove the slience indicator h# at the beginning and the end
+                labels = labels.replace('h# ', '')
                 
         return labels
     
